/* -----------------------------------------
  archio.hpp
  
  All the code in this file is open source
  so feel free to use it any way you would
  like.
  
  designed and implemented by
  ArchMiffo, member of Darkbits
----------------------------------------- */

#ifndef AUL_ARCHIO_HPP
#define AUL_ARCHIO_HPP

/* -------------------
  Include directives
------------------- */
#include <istream>
#include <limits>
#include <cctype>
#include <algorithm>

// aul stands for ArchMiffo Utility Library
namespace aul
{

/* ---------------------------------------------------
  The archio is a work in progress. What you see
  here is just some stuff that I did to help me
  when I wrote the console code for another project.
  Most of the functions are nothing more tham a
  wrapper around basic istream funtions. The idea
  whith theese wrappers are that you will get a 
  more flexible function that will work in more
  cases whithout having to think about the syntax.
  Hopefully I will add some more usefull functions
  to this library in the future since it ain't very
  interesting right now.
--------------------------------------------------- */

/* ----------------------------------
  IgnoreLine
  
  Ignores everything in the input
  stream until a linebreak is found
---------------------------------- */
template<class charT, class traits>
inline std::basic_istream<charT, traits>& 
IgnoreLine(std::basic_istream<charT, traits>& in)
{
  in.ignore(std::numeric_limits<int>::max(), in.widen('\n'));
  return in;
} // end IgnoreLine

/* ---------------------------------------
  IgnoreTo

  A more generell form of IgnoreLine.
  This function could be used everywhere
  you use IgnoreLine but will also work
  if you want to ignore everything to
  some other character that you specify.
--------------------------------------- */
template<class charT, class traits>
inline std::basic_istream<charT, traits>& 
IgnoreTo(std::basic_istream<charT, traits>& in, char delim)
{
  in.ignore(std::numeric_limits<int>::max(), in.widen(delim));
  return in;
} // end IgnoreLine

/* ----------------------------------------------
  GetNextChar
  
  What this funtion does is return the next
  char that will be read from the input stream.
  Unlike peek this function skips all the 
  leading whitespaces and returns an actual
  character (this is not all true becouse it
  could return a '\n' or something similar).
---------------------------------------------- */
template<class charT, class traits>
char GetNextChar(std::basic_istream<charT, traits>& in)
{
  in>>std::ws;
  return in.peek();
} // end GetNextChar

/* --------------------------------------------------
  GetLine
  
  This is a rather pathetic wrapper that lets you 
  get the a whole line up to a delimiter by writing 
  about 7 charactersless than whithout the wrapper. 
  The extra characters that you don't have to write 
  gives you a function that will work in almost any 
  case.
-------------------------------------------------- */
template<class charT, class traits>
inline std::basic_istream<charT, traits>& 
GetLine(std::basic_istream<charT, traits>& in, std::string& str, char delim)
{
  std::getline(in, str, in.widen(delim));
  return in;
} // end GetLine

/* -------------------------------------------
  RemoveSpace
  
  This function removes all space characters
  from a string
------------------------------------------- */
inline void RemoveSpace(std::string& str)
{
  str.erase(std::remove_if(str.begin(), str.end(), std::isspace), str.end());
} // end RemoveSpace

/* ---------------------------------------------------
  GetCommadVal
  
  This function was designed to be used as a part
  of a some config stuff I wrote. What it does is, 
  given an input stream, extracts a command and it's 
  value to the two strings that the user supplies. 
  Example: 
  This is in the stream: invertmouse = 1
  GetCommandVal(is, str1, str2, '=')
  => str1 = invertmouse, str2 = 1.
--------------------------------------------------- */
template<class charT, class traits>
std::basic_istream<charT, traits>&
GetCommandVal(std::basic_istream<charT, traits>& in, 
              std::string& command, std::string& val, char sep)
{
  std::string str;
  in>>std::ws;
  GetLine(in, str, '\n');
  
  RemoveSpace(str);

  std::string::size_type index = str.find(sep);
  if(index == std::string::npos)
    throw "wrong seperator";

  command = std::string(str.begin(), str.begin() + index);
  val = std::string(str.begin() + index + 1, str.end());

  return in;
} // end GetCommandVal

} // end aul

#endif // end AUL_ARCHIO_HPP